package it.unibo.tw.dao;

import java.util.List;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class ItemDAOTest {

	public static final int DAO = DAOFactory.HSQLDB;
	
	public ItemDAO itemDAO = null;

	public ItemDTO itemDTO = null;

	@Before
	public void createTable() {
		itemDAO = DAOFactory.getDAOFactory(DAO).getItemDAO();
		itemDAO.createTable();
		//
		itemDTO = new ItemDTO();
		itemDTO.setName("Art.1");
		itemDTO.setDescription("Bla bla bla");
		itemDTO.setPrice(22.50);
		itemDTO.setAvailability(100);
	}

	@After
	public void dropTable() {
		itemDAO.dropTable();
		itemDAO = null;
		itemDTO = null;
	}

	// --------

	@Test
	public void testReadNone() {
		Assert.assertEquals("No entries in the beginning",0,itemDAO.find(null).size());
	}

	@Test
	public void testCreateAndRead() {
		Long ItemDTO_id = itemDAO.create(itemDTO);
		Assert.assertTrue("Created entry got non negative id",ItemDTO_id >= 0);
		Assert.assertEquals("One entry was written to the database",1,itemDAO.find(null).size());
		Assert.assertTrue("Created entries are re-read correctly",itemDTO.equals(itemDAO.read(ItemDTO_id)));
	}

	@Test
	public void testUpdate() {
		Long ItemDTO_id = itemDAO.create(itemDTO);
		// la update accetta un intero DTO che deve essere già sul db! dobbiamo aggiungere l'id anche nel dato che abbiamo noi!
		itemDTO.setId(ItemDTO_id);
		itemDTO.setName("Art.1 bis");
		itemDTO.setDescription("Blah blah blah");
		itemDTO.setPrice(30.99);
		itemDTO.setAvailability(5);
		Assert.assertTrue("Update user was successful",itemDAO.update(itemDTO));
		Assert.assertTrue("Update verification was ok too",itemDTO.equals(itemDAO.read(ItemDTO_id)));
	}

	@Test
	public void testDelete() {
		Long ItemDTO_id = itemDAO.create(itemDTO);
		// la delete accetta un intero DTO, non solo l'id! quindi dobbiamo recuperare il DTO intero prima!
		ItemDTO ItemDTO_to_delete = itemDAO.read(ItemDTO_id);
		Assert.assertTrue("Correct deletion returns boolean result true",itemDAO.delete(ItemDTO_to_delete.getId()));
		Assert.assertEquals("No items left",0,itemDAO.find(null).size());
	}

	@Test
	public void testFind() {
		Long ItemDTO_id = itemDAO.create(itemDTO);
		// aggiungo un secondo item tanto per complicare le cose
		ItemDTO item2 = new ItemDTO();
		item2.setName( "Altro " );
		item2.setDescription( "novita' ");
		item2.setPrice(99.99);
		item2.setAvailability(10);
		itemDAO.create(item2);
		// affinché il confronto abbia successo dobbiamo aggiungere l'id anche nel dato che abbiamo noi!
		itemDTO.setId(ItemDTO_id);
		List<ItemDTO> foundItems = itemDAO.find("Art");
		Assert.assertEquals("Find items works properly",1,foundItems.size());
		Assert.assertTrue("Find items works properly",itemDTO.equals(foundItems.get(0)));
	}

	@Test
	public void testFind2() {
		Long ItemDTO_id = itemDAO.create(itemDTO);
		// aggiungo un secondo item tanto per complicare le cose
		ItemDTO item2 = new ItemDTO();
		item2.setName( "Art.2" );
		item2.setDescription( "novita' ");
		item2.setPrice(99.99);
		item2.setAvailability(10);
		itemDAO.create(item2);
		// affinché il confronto abbia successo dobbiamo aggiungere l'id anche nel dato che abbiamo noi!
		itemDTO.setId(ItemDTO_id);
		List<ItemDTO> foundItems = itemDAO.find("Art");
		Assert.assertEquals("Find items works properly",2,foundItems.size());
	}

	@Test
	public void testSomethingElse() {
		Assert.fail("Not yet implemented");
	}

}
