package it.unibo.tw.dao;

import it.unibo.tw.dao.hsqldb.HsqldbDAOFactory;
import it.unibo.tw.dao.mysql.MySqlDAOFactory;

public abstract class DAOFactory {

	// --- List of supported DAO types ---
	
	/**
	 * Numeric constant '1' corresponds to explicit Hsqldb choice
	 */
	public static final int HSQLDB = 1;

	/**
	 * Numeric constant '2' corresponds to explicit MySQL choice
	 */
	public static final int MYSQL = 2;

	
	// --- Actual factory method ---
	
	/**
	 * Depending on the input parameter
	 * this method returns one out of several possible 
	 * implementations of this factory spec 
	 */
	public static DAOFactory getDAOFactory(int whichFactory) {
		switch ( whichFactory ) {
		case HSQLDB:
			return new HsqldbDAOFactory();
		case MYSQL:
			return new MySqlDAOFactory();
		default:
			return null;
		}
	}
	
	
	
	// --- Factory specification: concrete factories implementing this spec must provide this methods! ---

	/**
	 * Method to obtain a DATA ACCESS OBJECT
	 * for the datatype 'Item'
	 */
	public abstract ItemDAO getItemDAO();

	/**
	 * Method to obtain a DATA ACCESS OBJECT
	 * for the datatype 'Order'
	 */
	public abstract OrderDAO getOrderDAO();

}
