package it.unibo.tw.dao;

import java.util.ArrayList;
import java.util.List;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class OrderDAOTest {

	public static final int DAO = DAOFactory.HSQLDB;
	
	public OrderDAO orderDAO = null;

	public OrderDTO orderDTO = null;

	public List<ItemDTO> db_items = new ArrayList<ItemDTO>();
	
	@Before
	public void createTable() {
		orderDAO = DAOFactory.getDAOFactory(DAO).getOrderDAO();
		orderDAO.createTable();
		//
		ItemDAO itemDAO = DAOFactory.getDAOFactory(DAO).getItemDAO();
		itemDAO.dropTable();
		itemDAO.createTable();
		for ( int i = 0 ; i < 3 ; i++ ) {
			ItemDTO db_item = new ItemDTO();
			db_item.setName("Art."+i);
			db_item.setDescription("description"+i);
			db_item.setPrice(9.99);
			db_item.setAvailability(100);
			db_item.setId( itemDAO.create(db_item) );
			db_items.add(db_item);
		}
		//
		orderDTO = new OrderDTO();
		orderDTO.setBuyer("techweb");
		orderDTO.setShipment("Lab4");
		orderDTO.setEmail("samuele.pasini@unibo.it");
		
		for ( ItemDTO item : db_items ) {
			OrderItemDTO oi = new OrderItemDTO();
			oi.setItemDTO(item);
			oi.setQuantity(10);
			orderDTO.setItems( new ArrayList<OrderItemDTO>() );
			orderDTO.getItems().add( oi );
		}
	}

	@After
	public void dropTable() {
		orderDAO.dropTable();
		orderDAO = null;
		orderDTO = null;
	}

	// --------

	@Test
	public void testReadNone() {
		Assert.assertEquals("No entries in the beginning",0,orderDAO.find(null).size());
	}

	@Test
	public void testCreateAndRead() {
		Long OrderDTO_id = orderDAO.create(orderDTO);
		Assert.assertTrue("Created entry got non negative id",OrderDTO_id >= 0);
		Assert.assertEquals("One entry was written to the database",1,orderDAO.find(null).size());
		Assert.assertEquals("Created entries are re-read correctly",orderDTO.getItems().size(), orderDAO.read(OrderDTO_id).getItems().size());
	}

	@Test
	public void testDelete() {
		Long OrderDTO_id = orderDAO.create(orderDTO);
		// la delete accetta un intero DTO, non solo l'id! quindi dobbiamo recuperare il DTO intero prima!
		OrderDTO OrderDTO_to_delete = orderDAO.read(OrderDTO_id);
		Assert.assertTrue("Correct deletion returns boolean result true",orderDAO.delete(OrderDTO_to_delete.getId()));
		Assert.assertEquals("No items left",0,orderDAO.find(null).size());
	}

	@Test
	public void testFind() {
		Long OrderDTO_id = orderDAO.create(orderDTO);
		// aggiungo un secondo item tanto per complicare le cose
		OrderDTO order2 = new OrderDTO();
		order2.setBuyer("buyer2" );
		order2.setShipment( "nowhere");
		order2.setItems( new ArrayList<OrderItemDTO>() );
		OrderItemDTO oi = new OrderItemDTO();
		oi.setItemDTO( db_items.get(0) );
		oi.setQuantity(5);
		order2.getItems().add(oi);
		orderDAO.create(order2);
		// affinché il confronto abbia successo dobbiamo aggiungere l'id anche nel dato che abbiamo noi!
		orderDTO.setId(OrderDTO_id);
		List<OrderDTO> foundOrders = orderDAO.find( orderDTO.getBuyer() );
		Assert.assertEquals("Find items works properly",1,foundOrders.size());
		// find does not fetch joined data!!
		Assert.assertNull("Find items does not fetch children",foundOrders.get(0).getItems() );
	}

	@Test
	public void testFind2() {
		Long OrderDTO_id = orderDAO.create(orderDTO);
		// aggiungo un secondo item tanto per complicare le cose
		OrderDTO order2 = new OrderDTO();
		order2.setBuyer( orderDTO.getBuyer() );
		order2.setShipment( "nowhere");
		order2.setItems( new ArrayList<OrderItemDTO>() );
		OrderItemDTO oi = new OrderItemDTO();
		oi.setItemDTO( db_items.get(0) );
		oi.setQuantity(5);
		order2.getItems().add(oi);
		orderDAO.create(order2);
		// affinché il confronto abbia successo dobbiamo aggiungere l'id anche nel dato che abbiamo noi!
		orderDTO.setId(OrderDTO_id);
		List<OrderDTO> foundOrders = orderDAO.find( orderDTO.getBuyer() );
		Assert.assertEquals("Find items works properly",2,foundOrders.size());
	}

	@Test
	public void testSomethingElse() {
		Assert.fail("Not yet implemented");
	}

}
